@@ -1,8 +1,9 @@
 from ploomber.clients.Client import Client
 from ploomber.clients.db import DBAPIClient, SQLAlchemyClient
 from ploomber.clients.shell import ShellClient, RemoteShellClient
+from ploomber.clients.gcloud import GCloudStorageClient
 
 __all__ = [
     'Client', 'DBAPIClient', 'SQLAlchemyClient', 'ShellClient',
-    'RemoteShellClient'
+    'RemoteShellClient', 'GCloudStorageClient'
 ]
