@@ -34,14 +34,13 @@ from copy import copy, deepcopy
 from pathlib import Path
 import warnings
 import logging
-import collections
 import tempfile
 from math import ceil
 from functools import partial
 
 try:
     import importlib.resources as importlib_resources
-except ImportError:
+except ImportError:  # pragma: no cover
     # backported
     import importlib_resources
 
@@ -59,14 +58,16 @@ from ploomber import executors
 from ploomber.constants import TaskStatus, DAGStatus
 from ploomber.exceptions import (DAGBuildError, DAGRenderError,
                                  DAGBuildEarlyStop)
-from ploomber.executors import Serial
-from ploomber.MessageCollector import MessageCollector
+from ploomber.MessageCollector import (RenderExceptionsCollector,
+                                       RenderWarningsCollector)
 from ploomber.util.util import callback_check
 from ploomber.dag.DAGConfiguration import DAGConfiguration
 from ploomber.dag.DAGLogger import DAGLogger
+from ploomber.dag.dagclients import DAGClients
+from ploomber.dag.abstractdag import AbstractDAG
 
 
-class DAG(collections.abc.Mapping):
+class DAG(AbstractDAG):
     """A collection of tasks with dependencies
 
     Parameters
@@ -106,6 +107,16 @@ class DAG(collections.abc.Mapping):
         "traceback" which will contain a dictionary, possible keys are "build"
         which contains the build error traceback and "on_finish" which contains
         the on_finish hook traceback, if any.
+
+    serializer : callable
+        Function to serialize products from PythonCallable tasks. Used if the
+        task has no serializer. See ``ploombe.tasks.PythonCallable``
+        documentation for details.
+
+    unserializer : callable
+        Function to unserialize products from PythonCallable tasks. Used if the
+        task has no serializer. See ``ploombe.tasks.PythonCallable``
+        documentation for details.
     """
     def __init__(self, name=None, clients=None, executor='serial'):
         self._G = nx.DiGraph()
@@ -113,14 +124,14 @@ class DAG(collections.abc.Mapping):
         self.name = name or 'No name'
         self._logger = logging.getLogger(__name__)
 
-        self._clients = clients or {}
+        self._clients = DAGClients(clients)
         self.__exec_status = DAGStatus.WaitingRender
 
         if executor == 'serial':
             self._executor = executors.Serial()
         elif executor == 'parallel':
             self._executor = executors.Parallel()
-        elif isinstance(executor, executors.Executor.Executor):
+        elif isinstance(executor, executors.abc.Executor):
             self._executor = executor
         else:
             raise TypeError(
@@ -299,8 +310,8 @@ class DAG(collections.abc.Mapping):
             else:
                 tasks = self.values()
 
-            exceptions = MessageCollector()
-            warnings_ = MessageCollector()
+            exceptions = RenderExceptionsCollector()
+            warnings_ = RenderWarningsCollector()
 
             # reset all tasks status
             for task in tasks:
@@ -323,30 +334,30 @@ class DAG(collections.abc.Mapping):
                             outdated_by_code=self._params.outdated_by_code)
                     except Exception:
                         tr = traceback.format_exc()
-                        exceptions.append(message=tr, task_str=repr(t))
+                        exceptions.append(task=t, message=tr)
 
                 if warnings_current:
                     w = [
                         str(a_warning.message)
                         for a_warning in warnings_current
                     ]
-                    warnings_.append(task_str=t.name, message='\n'.join(w))
+                    warnings_.append(task=t, message='\n'.join(w))
 
             if warnings_:
                 # FIXME: maybe raise one by one to keep the warning type
-                warnings.warn('Some tasks had warnings when rendering DAG '
-                              '"{}":\n{}'.format(self.name, str(warnings_)))
+                warnings.warn(str(warnings_))
 
             if exceptions:
                 self._exec_status = DAGStatus.ErroredRender
-                raise DAGRenderError('DAG render failed, the following '
-                                     'tasks could not render '
-                                     '(corresponding tasks aborted '
-                                     'rendering):\n{}'.format(str(exceptions)))
+                raise DAGRenderError(str(exceptions))
 
         self._exec_status = DAGStatus.WaitingExecution
 
-    def build(self, force=False, show_progress=True, debug=False):
+    def build(self,
+              force=False,
+              show_progress=True,
+              debug=False,
+              close_clients=True):
         """
         Runs the DAG in order so that all upstream dependencies are run for
         every task
@@ -362,9 +373,13 @@ class DAG(collections.abc.Mapping):
 
         debug : bool, default=False
             Drop a debugging session if building raises an exception. Note that
-            this modifies the executor and temporarily sets it to Serial
+            this modifies the executor, temporarily setting it to Serial
             with subprocess off and catching exceptions/warnings off. Restores
-            the original executor at the end.
+            the original executor at the end
+
+        close_clients : bool, default=True
+            Close all clients (dag-level, task-level and product-level) upon
+            successful build
 
         Notes
         -----
@@ -397,9 +412,9 @@ class DAG(collections.abc.Mapping):
         # the debugging session in the right place
         if debug:
             executor_original = self.executor
-            self.executor = Serial(build_in_subprocess=False,
-                                   catch_exceptions=False,
-                                   catch_warnings=False)
+            self.executor = executors.Serial(build_in_subprocess=False,
+                                             catch_exceptions=False,
+                                             catch_warnings=False)
 
         callable_ = partial(self._build,
                             force=force,
@@ -412,7 +427,8 @@ class DAG(collections.abc.Mapping):
                 else:
                     report = callable_()
             finally:
-                self._close_clients()
+                if close_clients:
+                    self.close_clients()
 
         if debug:
             self.executor = executor_original
@@ -420,9 +436,11 @@ class DAG(collections.abc.Mapping):
         return report
 
     def _build(self, force, show_progress):
-        # always render before building, the function might immediately
-        # return if the user turned render status caching on
-        self.render(force=force, show_progress=show_progress)
+        # always render before building (the function might immediately
+        # return if the user turned render status caching on)
+        # Do not show progress - should only be displayed when .render is
+        # called directly
+        self.render(force=force, show_progress=False)
 
         if self._exec_status == DAGStatus.ErroredRender:
             raise DAGBuildError('Cannot build dag that failed to render, '
@@ -448,9 +466,6 @@ class DAG(collections.abc.Mapping):
             # a debugging session at the line of failure)
             except DAGBuildError as e:
                 tb['build'] = traceback.format_exc()
-                # error build dag, log exception and set status
-                self._logger.exception('Failure when building DAG "{}"'.format(
-                    self.name))
                 self._exec_status = DAGStatus.Errored
                 build_exception = e
             except DAGBuildEarlyStop:
@@ -477,7 +492,7 @@ class DAG(collections.abc.Mapping):
                     tb['on_finish'] = traceback.format_exc()
                     # on_finish error, log exception and set status
                     msg = ('Exception when running on_finish '
-                           'for DAG "{}": {}'.format(self.name, e))
+                           'for DAG "{}"'.format(self.name))
                     self._logger.exception(msg)
                     self._exec_status = DAGStatus.Errored
 
@@ -509,10 +524,9 @@ class DAG(collections.abc.Mapping):
                         raise DAGBuildError(msg) from e
 
                 # on_failure hook executed, raise original exception
-                raise DAGBuildError(
-                    'Failed to build DAG {}'.format(self)) from build_exception
+                raise build_exception
 
-    def _close_clients(self):
+    def close_clients(self):
         """Close all clients (dag-level, task-level and product-level)
         """
         for client in self.clients.values():
@@ -604,22 +618,6 @@ class DAG(collections.abc.Mapping):
 
         return Table([self[name].status(**kwargs) for name in self])
 
-    def to_dict(self, include_plot=False):
-        """Returns a dict representation of the dag's Tasks,
-        only includes a few attributes.
-
-        Parameters
-        ----------
-        include_plot: bool, optional
-            If True, the path to a PNG file with the plot in "_plot"
-        """
-        d = {name: self._G.nodes[name]['task'].to_dict() for name in self._G}
-
-        if include_plot:
-            d['_plot'] = self.plot(open_image=False)
-
-        return d
-
     def to_markup(self, path=None, fmt='html', sections=None):
         """Returns a str (md or html) with the pipeline's description
 
@@ -709,8 +707,8 @@ class DAG(collections.abc.Mapping):
         """
         if task.name in self._G:
             raise ValueError('DAGs cannot have Tasks with repeated names, '
-                             'there is a Task with name "{}" '
-                             'already'.format(task.name))
+                             'there is a Task with name {} '
+                             'already'.format(repr(task.name)))
 
         if task.name is not None:
             self._G.add_node(task.name, task=task)
@@ -807,19 +805,19 @@ class DAG(collections.abc.Mapping):
         """
         Getting product status (outdated/up-to-date) is slow, especially for
         product whose metadata is stored remotely. This is critical when
-        rendering because we need to do a forward pass to know which tasks to run
-        and a product's status depends on its upstream product's status, and we
-        have to make sure we only retrieve metadata once, so we save a local
-        copy. But even with this implementation, we don't throw away product
-        status after rendering, otherwise calls that need project status
-        (like DAG.plot, DAG.status, DAG.to_markup) would have to get product
-        status before running its logic, so once we get it, we stick with it.
-        The only caveat is that status updates won't be reflected immediately
-        (e.g. if the user manually deletes a product's metadata), but that's
-        a small price to pay given that this is not expected to happen often.
-        The only case when we *must* be sure that we have up-to-date metadata
-        is when calling DAG.build(), so we call this method before building,
-        which forces metadata reload.
+        rendering because we need to do a forward pass to know which tasks to
+        run and a product's status depends on its upstream product's status,
+        and we have to make sure we only retrieve metadata once, so we save a
+        local copy. But even with this implementation, we don't throw away
+        product status after rendering, otherwise calls that need project
+        status (like DAG.plot, DAG.status, DAG.to_markup) would have to get
+        product status before running its logic, so once we get it, we stick
+        with it. The only caveat is that status updates won't be reflected
+        immediately (e.g. if the user manually deletes a product's metadata),
+        but that's a small price to pay given that this is not expected to
+        happen often. The only case when we *must* be sure that we have
+        up-to-date metadata is when calling DAG.build(), so we call this
+        method before building, which forces metadata reload.
         """
         self._logger.debug('Clearing product status')
         # clearing out this way is only useful after building, but not
@@ -872,9 +870,6 @@ class DAG(collections.abc.Mapping):
     def __repr__(self):
         return '{}("{}")'.format(type(self).__name__, self.name)
 
-    def _short_repr(self):
-        return repr(self)
-
     # IPython integration
     # https://ipython.readthedocs.io/en/stable/config/integrating.html
 
