@@ -3,13 +3,14 @@ import importlib
 import platform
 from pathlib import Path
 from collections.abc import Mapping
+from reprlib import Repr
 
 import yaml
 
 from ploomber.env import validate
 from ploomber.env.expand import EnvironmentExpander
 from ploomber.env.FrozenJSON import FrozenJSON
-from ploomber.util.util import find_file_recursively
+from ploomber.util import default
 
 
 # TODO: custom expanders, this could be done trough another special directive
@@ -25,25 +26,55 @@ class EnvDict(Mapping):
     source : dict or str
         If str, it will be interpreted as a path to a YAML file
 
+    path_to_here : str or pathlib.Path
+        Value used to expand the {{here}} placeholder. If None, it uses the
+        location of the YAML spec. If initialized with a dict and None,
+        the {{here}} placeholder is not available.
+
+    defaults : dict, default=None
+        Default values to use. If not None, it uses these as defaults and
+        overwrites keys using values in source
+
+    Notes
+    -----
+    By default, it includes the following placeholders (unless the passed
+    dictionary already contains those keys): {{user}} (current user)
+    {{cwd}} (working directory), {{here}} (env.yaml location, if any), {{root}}
+    (project's root folder, if any)
     """
-    def __init__(self, source):
+    def __init__(self, source, path_to_here=None, defaults=None):
+
         # if initialized from another EnvDict, copy the attributes to
         # initialize
         # this happens in the  CLI parser, which instanttiates the env
         # because it needs to create one and then replace cli args, then
         # passes this modified object to DAGSpec
         if isinstance(source, EnvDict):
-            for attr in ('_path_to_env', '_preprocessed', '_expander',
-                         '_data'):
+            for attr in (
+                    '_path_to_env',
+                    '_preprocessed',
+                    '_expander',
+                    '_data',
+                    '_repr',
+            ):
                 original = getattr(source, attr)
                 setattr(self, attr, deepcopy(original))
         else:
-            # load data
             (
+                # load data
                 raw_data,
                 # this will be None if source is a dict
                 self._path_to_env) = load_from_source(source)
 
+            if defaults:
+                raw_data = {**defaults, **raw_data}
+
+            # add default placeholders but override them if they are defined
+            # in the raw data
+            default = self._default_dict(include_here=path_to_here is not None)
+            self._default_keys = set(default) - set(raw_data)
+            raw_data = {**default, **raw_data}
+
             # check raw data is ok
             validate.raw_data_keys(raw_data)
 
@@ -52,13 +83,42 @@ class EnvDict(Mapping):
 
             # initialize expander, which converts placeholders to their values
             # we need to pass path_to_env since the {{here}} placeholder
-            # resolves
-            # to its parent
+            # resolves to its parent
+            if path_to_here is None:
+                # if no pat_to_here, use path_to_end
+                path_to_here = (None if self._path_to_env is None else Path(
+                    self._path_to_env).parent)
+            else:
+                path_to_here = Path(path_to_here).resolve()
+
             self._expander = EnvironmentExpander(self._preprocessed,
-                                                 self._path_to_env)
+                                                 path_to_here=path_to_here)
             # now expand all values
             self._data = self._expander.expand_raw_dictionary(raw_data)
 
+            self._repr = Repr()
+
+    @property
+    def default_keys(self):
+        """Returns keys whose default value is used
+        """
+        return self._default_keys
+
+    @staticmethod
+    def _default_dict(include_here):
+        placeholders = {
+            'user': '{{user}}',
+            'cwd': '{{cwd}}',
+        }
+
+        if default.find_root_recursively() is not None:
+            placeholders['root'] = '{{root}}'
+
+        if include_here:
+            placeholders['here'] = '{{here}}'
+
+        return placeholders
+
     @property
     def path_to_env(self):
         return self._path_to_env
@@ -106,7 +166,8 @@ class EnvDict(Mapping):
         return str(self._data)
 
     def __repr__(self):
-        return '{}({})'.format(type(self).__name__, str(self))
+        content = self._repr.repr_dict(self._data, level=2)
+        return f'{type(self).__name__}({content})'
 
     def _replace_value(self, value, keys_all):
         """
@@ -210,7 +271,7 @@ def load_from_source(source):
     elif isinstance(source, (str, Path)):
         # if not pointing to a file, try to locate it...
         if not Path(source).exists():
-            source_found = find_file_recursively(source)
+            source_found = default.find_file_recursively(source)
 
             if source_found is None:
                 raise FileNotFoundError('Could not find file "{}" in the '
@@ -276,22 +337,29 @@ def raw_preprocess(raw, path_to_raw):
                 raise ValueError('_module cannot be {{here}} if '
                                  'not loaded from a file')
         else:
-            try:
-                module_spec = importlib.util.find_spec(module)
-            except ValueError:
-                # raises ValueError if passed "."
-                module_spec = None
+            # check if it's a filesystem path
+            as_path = Path(module)
 
-            if not module_spec:
-                path_to_module = Path(module)
-
-                if not (path_to_module.exists() and path_to_module.is_dir()):
-                    raise ValueError('Could not resolve _module "{}", '
-                                     'failed to import as a module '
-                                     'and is not a directory'.format(module))
+            if as_path.exists():
+                if as_path.is_file():
+                    raise ValueError(
+                        'Could not resolve _module "{}", '
+                        'expected a module or a directory but got a '
+                        'file'.format(module))
+                else:
+                    path_to_module = as_path
 
+            # must be a dotted path
             else:
-                path_to_module = Path(module_spec.origin).parent
+                module_spec = importlib.util.find_spec(module)
+
+                # package does not exist
+                if module_spec is None:
+                    raise ValueError('Could not resolve _module "{}", '
+                                     'it is not a valid module '
+                                     'nor a directory'.format(module))
+                else:
+                    path_to_module = Path(module_spec.origin).parent
 
             preprocessed['_module'] = path_to_module
 
@@ -309,9 +377,9 @@ def find_env_w_name(name):
     path_to_env : pathlib.Path
         Path to environment file, None if no file could be found
     """
-    path = find_file_recursively(name='env.{}.yaml'.format(name))
+    path = default.find_file_recursively(name='env.{}.yaml'.format(name))
 
     if path is None:
-        return find_file_recursively(name='env.yaml')
+        return default.find_file_recursively(name='env.yaml')
     else:
         return path
