@@ -1,3 +1,4 @@
+import os
 import re
 import ast
 import pydoc
@@ -6,10 +7,11 @@ from copy import deepcopy, copy
 from collections.abc import Mapping
 from pathlib import Path
 
-from jinja2 import Template, StrictUndefined, UndefinedError
+from jinja2 import Template, StrictUndefined
 
 from ploomber.placeholders import util
 from ploomber import repo
+from ploomber.util import default
 
 
 def expand_raw_dictionary(raw, mapping):
@@ -25,6 +27,13 @@ def expand_raw_dictionary(raw, mapping):
     return data
 
 
+def expand_raw_dictionaries(raw, mapping):
+    """
+    Expands a list of dictionaries
+    """
+    return [expand_raw_dictionary(element, mapping) for element in raw]
+
+
 def expand_if_needed(raw_value, mapping):
     placeholders = util.get_tags_in_str(raw_value)
 
@@ -34,9 +43,15 @@ def expand_if_needed(raw_value, mapping):
         try:
             value = Template(raw_value,
                              undefined=StrictUndefined).render(**mapping)
-        except UndefinedError as e:
-            raise KeyError('Failed to replace placeholders: %s' %
-                           str(e)) from e
+        except Exception as e:
+            exception = e
+        else:
+            exception = None
+
+        if exception:
+            raise KeyError(
+                f'Error replacing placeholder: {exception}. Loaded env: '
+                f'{mapping!r}')
 
     return cast_if_possible(value)
 
@@ -47,10 +62,14 @@ def cast_if_possible(value):
     value to still have the appropriate type
     """
     if isinstance(value, str):
-        if value.lower() == 'false':
+        value_lower = value.lower()
+
+        if value_lower == 'false':
             return False
-        elif value.lower() == 'true':
+        elif value_lower == 'true':
             return True
+        elif value_lower in {'none', 'null'}:
+            return None
 
         try:
             return int(value)
@@ -67,18 +86,31 @@ def cast_if_possible(value):
 
 class EnvironmentExpander:
     """
-    Conver values in the raw dictionary by expanding tags such as {{git}},
+    Convert values in the raw dictionary by expanding tags such as {{git}},
     {{version}} or {{here}}. See `expand_raw_value` for more details
+
+    Parameters
+    ----------
+    preprocessed : dict
+        Preprocessed env dictionary
+
+    path_to_here : str
+        Path to env.yaml, used to expand {{here}}
+
+    version_requires_import : bool, default=False
+        Whether determining package version requires import or not. If False,
+        the root ``__init__.py`` file in the module must have
+        a ``__version__ = 'LITERAL'`` variable. Literal is extracted.
     """
     def __init__(self,
                  preprocessed,
-                 path_to_env=None,
+                 path_to_here=None,
                  version_requires_import=False):
         self._preprocessed = preprocessed
 
         # {{here}} resolves to this value
-        self._path_to_here = (None if path_to_env is None else str(
-            Path(path_to_env).parent))
+        self._path_to_here = (None if path_to_here is None else str(
+            Path(path_to_here).resolve()))
         # we compute every placeholder's value so we only do it once
         self._placeholders = {}
 
@@ -192,12 +224,25 @@ class EnvironmentExpander:
     def get_user(self):
         return getpass.getuser()
 
+    def get_cwd(self):
+        return str(Path(os.getcwd()).resolve())
+
+    def get_root(self):
+        root = default.find_root_recursively()
+
+        if root is None:
+            raise ValueError('Failed to expand {{root}}, could not '
+                             'find a setup.py in a parent folder')
+
+        return root
+
     def get_here(self):
         if self._path_to_here:
             return self._path_to_here
         else:
             raise RuntimeError('here placeholder is only available '
-                               'when env was initialized from a file')
+                               'when env was initialized from a file or '
+                               'when directly passing path to use')
 
     def get_git(self):
         module = self._preprocessed.get('_module')
