@@ -1,16 +1,12 @@
-from collections.abc import Iterable
 import pydoc
 from pathlib import Path
 from ploomber.placeholders.Placeholder import Placeholder
 from ploomber.placeholders import extensions
+from ploomber.util.util import isiterable_not_str
 
 from jinja2 import Environment, FileSystemLoader, StrictUndefined, exceptions
 
 
-def _is_iterable(o):
-    return isinstance(o, Iterable) and not isinstance(o, str)
-
-
 def _is_iterable_w_types(o, types):
     for element in o:
         if not isinstance(o, types):
@@ -51,7 +47,7 @@ class SourceLoader:
             raise TypeError('Path cannot be None if module is None')
 
         # validate path
-        if _is_iterable(path):
+        if isiterable_not_str(path):
             if _is_iterable_w_types(path, (str, Path)):
                 types_found = set(type(element) for element in path)
                 raise TypeError('If passing an iterable, path must consist '
@@ -80,7 +76,7 @@ class SourceLoader:
             raise ValueError('Could not find module path, pass a string or a '
                              'module object')
 
-        if _is_iterable(path):
+        if isiterable_not_str(path):
             self.path_full = [str(Path(module_path, e)) for e in path]
         else:
             # if path is None, do not append anything
@@ -117,7 +113,31 @@ class SourceLoader:
     def get_template(self, name):
         """Load a template by name
         """
-        template = self.env.get_template(str(name))
+        try:
+            template = self.env.get_template(str(name))
+        except exceptions.TemplateNotFound as e:
+            exception = e
+        else:
+            exception = None
+
+        if exception is not None:
+            expected_path = str(Path(self.path_full, name))
+
+            # user saved the template locally, but the source loader is
+            # configured to load from a different place
+            if Path(name).exists():
+                raise exceptions.TemplateNotFound(
+                    f'{str(name)!r} template does not exist. '
+                    'However such a file exists in the current working '
+                    'directory, if you want to load it as a template, move it '
+                    f'to {self.path_full!r} or remove the source_loader')
+            # no template and the file does not exist, raise a generic message
+            else:
+                raise exceptions.TemplateNotFound(
+                    f'{str(name)!r} template does not exist. '
+                    'Based on your configuration, if should be located '
+                    f'at: {expected_path!r}')
+
         return Placeholder(template)
 
     def _ipython_key_completions_(self):
