@@ -46,8 +46,9 @@ class GenericProduct(SQLiteBackedProductMixin, Product):
             default = self.task.dag.clients.get(type(self))
 
             if default is None:
-                raise ValueError('{} must be initialized with a client'.format(
-                    type(self).__name__))
+                raise ValueError(
+                    f'{type(self).__name__} must be initialized with a client.'
+                    ' Pass a client directly or a DAG-level one')
             else:
                 self._client = default
 
@@ -101,6 +102,12 @@ class GenericSQLRelation(GenericProduct):
     def kind(self):
         return self._identifier.kind
 
+    def __repr__(self):
+        return f'{type(self).__name__}({self._identifier._raw_repr()})'
+
+    def __hash__(self):
+        return hash((self.schema, self.name, self.kind))
+
 
 class SQLRelation(Product):
     """A product that represents a SQL relation but has no metadata
@@ -138,3 +145,9 @@ class SQLRelation(Product):
 
     def delete(self, force=False):
         pass
+
+    def __repr__(self):
+        return f'{type(self).__name__}({self._identifier._raw_repr()})'
+
+    def __hash__(self):
+        return hash((self.schema, self.name, self.kind))
