@@ -28,6 +28,10 @@ class Product(abc.ABC):
         any of the existing keys but additional key-value pairs might be
         included
     """
+
+    # TODO: previously, File didn't have a client parameter but it does now,
+    # it's best to include it here to simplify the constructors in the concrete
+    # classes
     def __init__(self, identifier):
         self._identifier = self._init_identifier(identifier)
 
@@ -174,10 +178,10 @@ class Product(abc.ABC):
         return str(self._identifier)
 
     def __repr__(self):
-        # NOTE: this assumes ._identifier has a best_str property,
+        # NOTE: this assumes ._identifier has a best_repr property,
         # should we refactor it?
         return '{}({})'.format(
-            type(self).__name__, self._identifier.best_str(shorten=True))
+            type(self).__name__, self._identifier.best_repr(shorten=True))
 
     def __getstate__(self):
         state = self.__dict__.copy()
@@ -207,15 +211,15 @@ class Product(abc.ABC):
 
     @abc.abstractmethod
     def _init_identifier(self, identifier):
-        pass
+        pass  # pragma: no cover
 
     @abc.abstractmethod
     def fetch_metadata(self):
-        pass
+        pass  # pragma: no cover
 
     @abc.abstractmethod
     def save_metadata(self, metadata):
-        pass
+        pass  # pragma: no cover
 
     @abc.abstractmethod
     def exists(self):
@@ -223,13 +227,13 @@ class Product(abc.ABC):
         This method returns True if the product exists, it is not part
         of the metadata, so there is no cached status
         """
-        pass
+        pass  # pragma: no cover
 
     @abc.abstractmethod
     def delete(self, force=False):
         """Deletes the product
         """
-        pass
+        pass  # pragma: no cover
 
     # NOTE: currently optional but there is a conflict with this. metadata
     # defines a delete public method which calls product._delete_metadata
@@ -238,3 +242,12 @@ class Product(abc.ABC):
         raise NotImplementedError(
             '_delete_metadata not implemented in {}'.format(
                 type(self).__name__))
+
+    # download and upload are only relevant for File but we add them to keep
+    # the API consistent
+
+    def download(self):
+        pass  # pragma: no cover
+
+    def upload(self):
+        pass  # pragma: no cover
