@@ -1,3 +1,6 @@
+"""
+Products representing SQL relations
+"""
 import abc
 import sqlite3
 import json
@@ -29,8 +32,9 @@ class ProductWithClientMixin:
             default = self.task.dag.clients.get(type(self))
 
             if default is None:
-                raise ValueError('{} must be initialized with a client'.format(
-                    type(self).__name__))
+                raise ValueError(
+                    f'{type(self).__name__} must be initialized with a client.'
+                    ' Pass a client directly or set a DAG-level one')
             else:
                 self._client = default
 
@@ -209,6 +213,15 @@ class SQLiteRelation(SQLiteBackedProductMixin, Product):
     def kind(self):
         return self._identifier.kind
 
+    def __eq__(self, other):
+        return self._identifier == other
+
+    def __hash__(self):
+        return hash((self.schema, self.name, self.kind))
+
+    def __repr__(self):
+        return f'{type(self).__name__}({self._identifier._raw_repr()})'
+
 
 # FIXME: self._identifier should not be accessed direclty since it might
 # be a placeholder
@@ -342,3 +355,12 @@ class PostgresRelation(ProductWithClientMixin, Product):
     @property
     def kind(self):
         return self._identifier.kind
+
+    def __eq__(self, other):
+        return self._identifier == other
+
+    def __hash__(self):
+        return hash((self.schema, self.name, self.kind))
+
+    def __repr__(self):
+        return f'{type(self).__name__}({self._identifier._raw_repr()})'
