@@ -18,5 +18,5 @@ variable in this file */
 change this */
 DROP TABLE IF EXISTS {{product}};
 
-CREATE TABLE AS {{product}}
-SELECT * FROM {{upstream['some_task']}};
\ No newline at end of file
+CREATE TABLE {{product}} AS
+SELECT * FROM {{upstream['some_task']}};
