@@ -7,9 +7,11 @@ from pathlib import Path
 from collections.abc import MutableMapping, Mapping
 
 from ploomber import tasks, products
-from ploomber.util.util import (load_dotted_path, _make_iterable,
-                                locate_dotted_path)
-from ploomber.util import validate
+from ploomber.util.util import _make_iterable
+from ploomber.util import validate, dotted_path
+from ploomber import validators
+
+from ploomber.exceptions import DAGSpecInitializationError
 
 suffix2taskclass = {
     '.py': tasks.NotebookRunner,
@@ -22,21 +24,34 @@ suffix2taskclass = {
 }
 
 
-def task_class_from_source_str(source_str, lazy_import):
+def _safe_suffix(product):
+    try:
+        return Path(product).suffix
+    except Exception:
+        return None
+
+
+def task_class_from_source_str(source_str, lazy_import, reload, product):
     """
     The source field in a DAG spec is a string. The actual value needed to
     instantiate the task depends on the task class, but to make task class
     optional, we try to guess the appropriate task here. If the source_str
     needs any pre-processing to pass it to the task constructor, it also
-    happens here
+    happens here. If product is not None, it's also used to determine if
+    a task is a SQLScript or SQLDump
     """
     extension = Path(source_str).suffix
 
     # if lazy load, just locate the module without importing it
-    fn_checker = locate_dotted_path if lazy_import else partial(
-        load_dotted_path, raise_=True)
+    fn_checker = dotted_path.locate_dotted_path if lazy_import else partial(
+        dotted_path.load_dotted_path, raise_=True, reload=reload)
 
     if extension and extension in suffix2taskclass:
+        if extension == '.sql' and _safe_suffix(product) in {
+                '.csv', '.parquet'
+        }:
+            return tasks.SQLDump
+
         return suffix2taskclass[extension]
     else:
         try:
@@ -47,18 +62,17 @@ def task_class_from_source_str(source_str, lazy_import):
             error = e
 
         if imported is None:
-            raise ValueError('Could not find an appropriate task class for '
-                             'source "{}", verify that the value is either '
-                             'a script with a valid extension or a valid '
-                             'dotted path (got error: "{}" when attempted to '
-                             ' import it). Alternatively, '
-                             'pass an explicit task class using the "class" '
-                             'key'.format(source_str, error))
+            raise ValueError(
+                'Could not determine task class for '
+                f'source {source_str!r}. This looks like a dotted path '
+                'but it failed to import. Original error '
+                f'message: {error!s}. You can also set the task class '
+                'using the "class" key')
         else:
             return tasks.PythonCallable
 
 
-def task_class_from_spec(task_spec, lazy_import):
+def task_class_from_spec(task_spec, lazy_import, reload):
     """
     Returns the class for the TaskSpec, if the spec already has the class
     name (str), it just returns the actual class object with such name,
@@ -67,9 +81,19 @@ def task_class_from_spec(task_spec, lazy_import):
     class_name = task_spec.get('class', None)
 
     if class_name:
-        class_ = getattr(tasks, class_name)
+        try:
+            class_ = validators.string.validate_task_class_name(class_name)
+        except Exception as e:
+            msg = f'Error validating Task spec (class field): {e.args[0]}'
+            e.args = (msg, )
+            raise
     else:
-        class_ = task_class_from_source_str(task_spec['source'], lazy_import)
+        class_ = task_class_from_source_str(
+            task_spec['source'],
+            lazy_import,
+            reload,
+            task_spec.get('product'),
+        )
 
     return class_
 
@@ -80,10 +104,14 @@ def source_for_task_class(source_str, task_class, project_root, lazy_import,
         if lazy_import:
             return source_str
         else:
-            return load_dotted_path(source_str)
+            return dotted_path.load_dotted_path(source_str)
     else:
         path = Path(source_str)
 
+        # NOTE: there is some inconsistent behavior here. project_root
+        # will be none if DAGSpec was initialized with a dictionary, hence
+        # this won't resolve to absolute paths - this is a bit confusing.
+        # maybe always convert to absolute?
         if project_root and not path.is_absolute() and make_absolute:
             return Path(project_root, source_str)
         else:
@@ -104,22 +132,32 @@ class TaskSpec(MutableMapping):
     project_root : str or pathlib.Path
         The project root folder (pipeline.yaml parent)
     lazy_import : bool, default=False
-            If False, sources are loaded when initializing the spec (e.g.
-            a dotted path is imported, a source loaded using a SourceLoader
-            is converted to a Placeholder object)
+        If False, sources are loaded when initializing the spec (e.g.
+        a dotted path is imported, a source loaded using a SourceLoader
+        is converted to a Placeholder object)
+    reload : bool, default=False
+        Reloads modules before getting dotted paths. Has no effect if
+        lazy_import=True
     """
-    def __init__(self, data, meta, project_root, lazy_import=False):
+    def __init__(self,
+                 data,
+                 meta,
+                 project_root,
+                 lazy_import=False,
+                 reload=False):
         # FIXME: make sure data and meta are immutable structures
         self.data = data
         self.meta = meta
         self.project_root = project_root
+        self.lazy_import = lazy_import
 
         self.validate()
 
         source_loader = meta['source_loader']
 
         # initialize required elements
-        self.data['class'] = task_class_from_spec(self.data, lazy_import)
+        self.data['class'] = task_class_from_spec(self.data, lazy_import,
+                                                  reload)
         # preprocess source obj, at this point it will either be a Path if the
         # task requires a file or a callable if it's a PythonCallable task
         self.data['source'] = source_for_task_class(
@@ -131,10 +169,13 @@ class TaskSpec(MutableMapping):
             # otherwise keep them relative
             make_absolute=source_loader is None)
 
-        is_a_file = isinstance(self.data['source'], Path)
+        is_path = isinstance(self.data['source'], Path)
 
-        if source_loader and is_a_file:
-            # if there is a source loader, use it...
+        # check if we need to use the source_loader. we don't if the path is
+        # relative because that doesn't sense with a source_loader.
+        # furthermore, this allows the user to load some sources from
+        # arbitrary locations
+        if source_loader and is_path and not self.data['source'].is_absolute():
             if lazy_import:
                 self.data['source'] = source_loader.path_to(
                     self.data['source'])
@@ -156,7 +197,7 @@ class TaskSpec(MutableMapping):
         validate.keys(valid=None,
                       passed=self.data,
                       required=required,
-                      name='task spec')
+                      name=repr(self))
 
         if self.meta['extract_upstream'] and self.data.get('upstream'):
             raise ValueError('Error validating task "{}", if '
@@ -189,20 +230,42 @@ class TaskSpec(MutableMapping):
         on_render = task_dict.pop('on_render', None)
         on_failure = task_dict.pop('on_failure', None)
 
-        task = class_(source=source,
-                      product=product,
-                      name=name,
-                      dag=dag,
-                      **task_dict)
+        if 'serializer' in task_dict:
+            task_dict['serializer'] = dotted_path.load_callable_dotted_path(
+                task_dict['serializer'])
+
+        if 'unserializer' in task_dict:
+            task_dict['unserializer'] = dotted_path.load_callable_dotted_path(
+                task_dict['unserializer'])
+
+        # edge case: if using lazy_import, we should not check if the kernel
+        # is installed. this is used when exporting to Argo/Airflow using
+        # soopervisor, since the exporting process should not require to have
+        # the ir kernel installed. The same applies when Airflow has to convert
+        # the DAG, the Airflow environment shouldn't require the ir kernel
+        if (class_ == tasks.NotebookRunner and self.lazy_import
+                and 'check_if_kernel_installed' not in task_dict):
+            task_dict['check_if_kernel_installed'] = False
+
+        try:
+            task = class_(source=source,
+                          product=product,
+                          name=name,
+                          dag=dag,
+                          **task_dict)
+        except Exception as e:
+            msg = f'Error initializing Task from {self!r}. Error: {e.args[0]}'
+            e.args = (msg, )
+            raise
 
         if on_finish:
-            task.on_finish = load_dotted_path(on_finish)
+            task.on_finish = dotted_path.load_callable_dotted_path(on_finish)
 
         if on_render:
-            task.on_render = load_dotted_path(on_render)
+            task.on_render = dotted_path.load_callable_dotted_path(on_render)
 
         if on_failure:
-            task.on_failure = load_dotted_path(on_failure)
+            task.on_failure = dotted_path.load_callable_dotted_path(on_failure)
 
         return task, upstream
 
@@ -249,9 +312,9 @@ def init_product(task_dict, meta, task_class, root_path):
     meta_product_default_class = get_value_at(meta, key)
 
     if 'product_class' in task_dict:
-        CLASS = getattr(products, task_dict.pop('product_class'))
+        CLASS = validate_product_class_name(task_dict.pop('product_class'))
     elif meta_product_default_class:
-        CLASS = getattr(products, meta_product_default_class)
+        CLASS = validate_product_class_name(meta_product_default_class)
     else:
         raise ValueError('Could not determine a product class for task: '
                          '"{}". Add an explicit value in the '
@@ -260,8 +323,9 @@ def init_product(task_dict, meta, task_class, root_path):
                          'key to the applicable task class'.format(task_dict))
 
     if 'product_client' in task_dict:
-        dotted_path = task_dict.pop('product_client')
-        kwargs = {'client': load_dotted_path(dotted_path)()}
+        kwargs = {
+            'client': dotted_path.call_spec(task_dict.pop('product_client'))
+        }
     else:
         kwargs = {}
 
@@ -270,24 +334,60 @@ def init_product(task_dict, meta, task_class, root_path):
 
     if isinstance(product_raw, Mapping):
         return {
-            key: CLASS(resolve_product(value, relative_to, CLASS), **kwargs)
+            key: try_product_init(CLASS,
+                                  resolve_if_file(value, relative_to, CLASS),
+                                  kwargs)
             for key, value in product_raw.items()
         }
     else:
-        return CLASS(resolve_product(product_raw, relative_to, CLASS),
-                     **kwargs)
+        source = resolve_if_file(product_raw, relative_to, CLASS)
+        return try_product_init(CLASS, source, kwargs)
 
 
-def resolve_product(product_raw, relative_to, class_):
+def try_product_init(class_, source, kwargs):
+    """
+    Try to initialize product, raises a chained exception if not possible.
+    This provides more contextual information
+    """
+    try:
+        return class_(source, **kwargs)
+    except Exception as e:
+        kwargs_msg = f'and keyword arguments: {kwargs!r}' if kwargs else ''
+        raise DAGSpecInitializationError(
+            f'Error initializing {class_.__name__} with source: '
+            f'{source!r}' + kwargs_msg) from e
+
+
+def validate_product_class_name(product_class_name):
+    try:
+        return validators.string.validate_product_class_name(
+            product_class_name)
+    except Exception as e:
+        msg = ('Error validating Task spec (product_class field): '
+               f'{e.args[0]}')
+        e.args = (msg, )
+        raise
+
+
+def resolve_if_file(product_raw, relative_to, class_):
+    try:
+        return _resolve_if_file(product_raw, relative_to, class_)
+    except Exception as e:
+        e.args = ('Error initializing File with argument '
+                  f'{product_raw!r} ({e})', )
+        raise
+
+
+def _resolve_if_file(product_raw, relative_to, class_):
     if class_ != products.File:
         return product_raw
     elif relative_to:
-        # When a DAG is initialized, paths are usually relative to the folder
-        # that has the pipeline.yaml, the only case when this isn't true is
-        # when using DAGSpec.auto_load(), in such case, relative paths won't
-        # work if the current working directory is different  to the
-        # pipeline.yaml folder (this happens sometimes in the Jupyter UI).
-        # We resolve paths to avoid ambiguity on this
+        # To keep things consistent, product relative paths are so to the
+        # pipeline.yaml file (not to the current working directory). This is
+        # important because there is no guarantee that the process calling
+        # this will be at the pipeline.yaml location. One example is
+        # when using the integration with Jupyter notebooks, each notebook
+        # will set its working directory to the current parent.
         return str(Path(relative_to, product_raw).resolve())
     else:
         return Path(product_raw).resolve()
@@ -295,8 +395,7 @@ def resolve_product(product_raw, relative_to, class_):
 
 def _init_client(task_dict):
     if 'client' in task_dict:
-        dotted_path = task_dict.pop('client')
-        task_dict['client'] = load_dotted_path(dotted_path)()
+        task_dict['client'] = dotted_path.call_spec(task_dict.pop('client'))
 
 
 def get_value_at(d, dotted_path):
