@@ -1,7 +1,7 @@
 from pathlib import Path
 from io import StringIO
 
-from ploomber.tasks.Task import Task
+from ploomber.tasks.abc import Task
 from ploomber.sources import (SQLScriptSource, SQLQuerySource, FileSource)
 from ploomber.products import (File, PostgresRelation, SQLiteRelation,
                                GenericSQLRelation, GenericProduct, SQLRelation)
@@ -46,19 +46,38 @@ class SQLScript(Task):
                  params=None):
         params = params or {}
 
-        kwargs = dict(hot_reload=dag._params.hot_reload)
+        client = client or dag.clients.get(type(self))
+
+        if client is None:
+            raise ValueError(
+                f'{type(self).__name__} must be initialized with a client. '
+                'Pass a client directly or set a DAG-level one')
+
+        if not hasattr(client, 'split_source'):
+            raise TypeError(
+                f'client with value {client!r} does not have a split_source '
+                'attribute. Make sure this is a valid client object '
+                '(e.g., ploomber.clients.SQLALChemy or '
+                'ploomber.clients.DBAPIClient)')
+
+        kwargs = dict(hot_reload=dag._params.hot_reload,
+                      split_source=client.split_source)
+
         self._source = type(self)._init_source(source, kwargs)
         super().__init__(product, dag, name, params)
-
-        self.client = client or self.dag.clients.get(type(self))
-
-        if self.client is None:
-            raise ValueError('{} must be initialized with a client'.format(
-                type(self).__name__))
+        self.client = client
+        self.dag = dag
 
     def run(self):
         return self.client.execute(str(self.source))
 
+    def load(self, limit=10):
+        """Load this task's product in a pandas.DataFrame
+        """
+        import pandas as pd
+        return pd.read_sql(f'SELECT * FROM {self.product} LIMIT {int(limit)}',
+                           self.client)
+
     @staticmethod
     def _init_source(source, kwargs):
         return SQLScriptSource(source, **kwargs)
@@ -90,11 +109,11 @@ class SQLDump(Task):
         refer to jinja2 documentation for details
     chunksize: int, optional
         Number of rows per file, otherwise download the entire dataset in
-        a single one. If set, the product will be a folder
+        a single one. If not None, the product becomes a directory
     io_handler: ploomber.io.CSVIO or ploomber.io.ParquetIO, optional
         io handler to use (which controls the output format), currently
         only csv and parquet are supported. If None, it tries to infer the
-        handler from the product's extension if that doesn't work, if uses
+        handler from the product's extension if that doesn't work, it uses
         io.CSVIO
 
     Notes
@@ -134,8 +153,9 @@ class SQLDump(Task):
             self.io_handler = io_handler
 
         if self.client is None:
-            raise ValueError('{} must be initialized with a client'.format(
-                type(self).__name__))
+            raise ValueError(
+                f'{type(self).__name__} must be initialized with a client. '
+                'Pass a client directly or set a DAG-level one')
 
     @staticmethod
     def _init_source(source, kwargs):
