@@ -11,21 +11,21 @@ import functools
 
 from IPython.terminal.debugger import TerminalPdb, Pdb
 
-from ploomber.tasks.Task import Task
+from ploomber.tasks.abc import Task
 from ploomber.sources import (PythonCallableSource, GenericSource, EmptySource)
 from ploomber.clients import ShellClient
 from ploomber.products.Metadata import MetadataAlwaysUpToDate
 from ploomber.exceptions import TaskBuildError
 from ploomber.constants import TaskStatus
 from ploomber.sources.interact import CallableInteractiveDeveloper
-from ploomber.tasks.Params import Params
+from ploomber.tasks._params import Params
 
 
 def _unserialize_params(params_original, unserializer):
     params = params_original.to_dict()
 
     params['upstream'] = {
-        k: unserializer(v)
+        k: unserializer(product=v)
         for k, v in params['upstream'].items()
     }
 
@@ -62,7 +62,9 @@ class PythonCallable(Task):
         A callable to serialize this task's product, must take two arguments,
         the first argument passed is the value returned by the task's
         source, the second argument is the product oject. If None, the
-        task's source is responsible for serializing its own product
+        task's source is responsible for serializing its own product. If
+        used, the source function must not have a "product" parameter but
+        return its result instead
     """
     def __init__(self,
                  source,
@@ -72,10 +74,10 @@ class PythonCallable(Task):
                  params=None,
                  unserializer=None,
                  serializer=None):
-        kwargs = dict(hot_reload=dag._params.hot_reload,
-                      needs_product=serializer is None)
-        self._source = type(self)._init_source(source, kwargs)
         self._serializer = serializer or dag.serializer
+        kwargs = dict(hot_reload=dag._params.hot_reload,
+                      needs_product=self._serializer is None)
+        self._source = type(self)._init_source(source, kwargs)
         self._unserializer = unserializer or dag.unserializer
         super().__init__(product, dag, name, params)
 
@@ -219,6 +221,7 @@ class PythonCallable(Task):
         return self._unserializer(str(self.product))
 
 
+# FIXME: there is already a TaskFactory, this is confusing
 def task_factory(_func=None, **factory_kwargs):
     """Syntactic sugar for building PythonCallable tasks
     """
@@ -319,30 +322,6 @@ class DownloadFromURL(Task):
         return GenericSource(str(source), **kwargs, optional=['product'])
 
 
-# TODO: move this and _Gather to helpers.py (actually create a module
-# for partitioned execution)
-class _Partition(Task):
-    def __init__(self, product, dag, name):
-        kwargs = dict(hot_reload=dag._params.hot_reload)
-        self._source = type(self)._init_source(kwargs)
-        super().__init__(product, dag, name, None)
-
-    def run(self):
-        """This Task does not run anything
-        """
-        # TODO: verify that this task has only one upstream dependencies
-        # is this the best place to check?
-        pass
-
-    def _init_source(self, kwargs):
-        return EmptySource(None, **kwargs)
-
-
-class _Gather(PythonCallable):
-    def run(self):
-        pass
-
-
 class Link(Task):
     """
     A dummy Task used to "plug" an external Product to a pipeline, this
